function Om = error_AWGN_NA_LB(n, R, Pe)
%
% Om = error_AWGN_NA_LB(n, R, Pe)
%
% inputs :
%  n    : packet length
%  R    : code rate
%  Pe   : reference packet error probability
%
% these must be vectors of the same length
%
% outputs:
%  Om    : SNR needed to match the normal approximation bound on error 
%          probability
%
% The method used builds upon the normal approximation, e.g., see 
% T. Erseghe, "Coding in the Finite-Blocklength Regime: Bounds based on 
% Laplace Integrals and their Asymptotic Approximations", 
% http://arxiv.org/abs/1511.04629
%
% (c) T. Erseghe 2016


q = @(x) 0.5*erfc((x)/sqrt(2));

for k = 1:length(n)
    if (n(k)==inf)
        Om(k) = 2^(2*R(k))-1;
    else
        ra = R(k);
        pe = Pe(k);
        Qe = fminsearch(@(x) abs(q(x)-pe),0,optimset('TolX',1e-10,'TolFun',1e-10));
        V = @(x) x.*(2+x)/2./((1+x).^2);
        C = @(x) log2(1+x)/2;
        Ra = @(x) C(x)-sqrt(V(x)/n(k))*log2(exp(1))*Qe+log2(n(k))/2/n(k);
        Om(k) = fminsearch(@(x) abs(Ra(x)-ra),1,optimset('TolX',1e-10,'TolFun',1e-10));
    end
end

