function R = rate_AWGN_NA_UB(n, Om, Pe)
%
% R = rate_AWGN_NA_UB(n, Om, Pe)
%
% inputs :
%  n    : packet length
%  Om   : SNR value
%  Pe   : reference packet error probability
%
% must be vectors of the same length 
%
% outputs:
%  R    : normal approximation bound on rate
%
% The method used builds upon the normal approximation, e.g., see 
% T. Erseghe, "Coding in the Finite-Blocklength Regime: Bounds based on 
% Laplace Integrals and their Asymptotic Approximations", 
% http://arxiv.org/abs/1511.04629
%
% (c) T. Erseghe 2016


q = @(x) 0.5*erfc((x)/sqrt(2));

for k = 1:length(n)
    x = Om(k);
    C = log2(1+x)/2;
    if (n(k)==inf)
        R(k) = C;
    else
        Qe = fminsearch(@(x) abs(q(x)-Pe(k)),0,optimset('TolX',1e-10,'TolFun',1e-10));
        V = x.*(2+x)/2./((1+x).^2);
        R(k) = C-sqrt(V/n(k))*log2(exp(1))*Qe+log2(n(k))/2/n(k);
        if (R(k)<0)||(~isreal(R(k)))
                    R(k) = nan;
        end
    end
end

