%% 2) PACKET ERROR RATE BOUNDS
disp(' ')
disp('PACKET ERROR RATE BOUNDS')

n = [5e2 1e3 2e3 5e3 1e4 1e5 1e6];
Pe = 10.^(-2:-0.5:-7);
uno = ones(size(Pe));
R = 1/2;
tic

% NA bound 
% expected execution time 68.0 sec
for k = 1:length(n)    
    Om_NA(k,:) = 10*log10(error_BIAWGN_NA_LB(n(k)*uno, R*uno, Pe));
end; toc; tic
% PPV bound 
% expected execution time 526 sec (9 min)
for k = 1:length(n)    
    Om_PPV(k,:) = 10*log10(error_BIAWGN_PPV_On2_LB(n(k)*uno, R*uno, Pe));
end; toc; tic

figure(1)
set(0,'defaulttextinterpreter','latex')
subplot(2,2,2)
semilogy(Om_PPV,Pe)
hold on
semilogy(Om_NA,Pe,'--')
hold off
xlabel('SNR $\Gamma$')
ylabel('error probability $P_e$')
grid


%% 3) OTHER PACKET ERROR RATE BOUNDS
disp(' ')
disp('OTHER PACKET ERROR RATE BOUNDS')

n = [5580, 1024, 500, 300];
R = [4093, 768, 400, 270]./n;

tic

% NA bound 
% expected execution time 28.6 sec
for k = 1:length(n)    
    Om_NA2(k,:) = 10*log10(error_BIAWGN_NA_LB(n(k)*uno, R(k)*uno, Pe)/(2*R(k)));
end; toc; tic
% PPV bound 
% expected execution time 336 sec (5 min)
for k = 1:length(n)    
    Om_PPV2(k,:) = 10*log10(error_BIAWGN_PPV_On2_LB(n(k)*uno, R(k)*uno, Pe)/(2*R(k)));
end; toc; tic

subplot(2,2,3)
semilogy(Om_PPV2,Pe)
hold on
semilogy(Om_NA2,Pe,'--')
hold off
xlabel('SNR $\Gamma$')
ylabel('error probability $P_e$')
grid


%% 3) AND OTHER PACKET ERROR RATE BOUNDS
disp(' ')
disp('AND OTHER PACKET ERROR RATE BOUNDS')

n = [1e4, 2640];
R = 1/2;

tic

% NA bound 
% expected execution time 18.9 sec
for k = 1:length(n)    
    Om_NA3(k,:) = 10*log10(error_BIAWGN_NA_LB(n(k)*uno, R*uno, Pe));
end; toc; tic
% PPV bound 
% expected execution time 140 sec (2 min)
for k = 1:length(n)    
    Om_PPV3(k,:) = 10*log10(error_BIAWGN_PPV_On2_LB(n(k)*uno, R*uno, Pe));
end; toc; tic

subplot(2,2,4)
semilogy(Om_PPV3,Pe)
hold on
semilogy(Om_NA3,Pe,'--')
hold off
xlabel('SNR $\Gamma$')
ylabel('error probability $P_e$')
grid


%% PRINT RESULT
% print -dpdf fig7b.pdf 
