

%% 1) SPECTRAL EFFICIENCY CURVES
disp(' ')
disp('SPECTRAL EFFICIENCY CURVES')

Pe = 1e-5;
data = {... % n + SNRs for: om_NA, om_RCU, om_PPV
    {{2e2},{-9:12},{0:12},{-3:12}},...
    {{5e2},{-9:12},{-5:12},{-6:12}},...
    {{1e3},{-10:12},{-8:12},{-9:12}},...
    {{2e3},{-15:12},{-11:12},{-13:12}},...
    {{5e3},{-17:12},{-14:12},{-15:12}},...
    {{1e4},{-20:12},{-17:12},{-18:12}},...
    {{1e5},{-28:12},{-27:12},{-28:12}},...
    {{1e6},{-32:12},{-30:12},{-32:12}}...
};
snr_dB = -32:15;

% PPV bound 
% expected execution time 20.8 sec (106 with kappa=40)
rho_PPV = nan(length(data),length(snr_dB));
EbN0_PPV = nan(length(data),length(snr_dB)); tic
kappa = 2;
for k=1:length(data)
    n = data{k}{1}{1};
    Om_dB = data{k}{4}{1};
    uno = ones(size(Om_dB));
    R_PPV = rate_BSC_PPV_OnX_UB(n*uno, 10.^(Om_dB/10), Pe*uno, kappa); 
    rho_PPV(k,ismember(snr_dB,Om_dB)) = 2*R_PPV;
    EbN0_PPV(k,ismember(snr_dB,Om_dB)) = Om_dB-10*log10(2*R_PPV);
end; toc

% RCU bound 
% expected execution time 16.7 sec
rho_RCU = nan(length(data),length(snr_dB));
EbN0_RCU = nan(length(data),length(snr_dB));  tic
for k=1:length(data)
    n = data{k}{1}{1};
    Om_dB = data{k}{3}{1};
    uno = ones(size(Om_dB));
    R_RCU = rate_BSC_RCU_On2_LB(n*uno, 10.^(Om_dB/10), Pe*uno); 
    rho_RCU(k,ismember(snr_dB,Om_dB)) = 2*R_RCU;
    EbN0_RCU(k,ismember(snr_dB,Om_dB)) = Om_dB-10*log10(2*R_RCU);
end; toc

% NA bound 
% expected execution time 1.8 sec
rho_NA = nan(length(data),length(snr_dB));
EbN0_NA = nan(length(data),length(snr_dB));  tic
for k=1:length(data)
    n = data{k}{1}{1};
    Om_dB = data{k}{2}{1};
    uno = ones(size(Om_dB));
    R_NA = rate_BSC_NA_UB(n*uno, 10.^(Om_dB/10), Pe*uno); 
    rho_NA(k,ismember(snr_dB,Om_dB)) = 2*R_NA;
    EbN0_NA(k,ismember(snr_dB,Om_dB)) = Om_dB-10*log10(2*R_NA);
end; toc


figure(1)
set(0,'defaulttextinterpreter','latex')
legend('interpreter','latex')
subplot(1,1,1)
semilogy(EbN0_PPV.',rho_PPV.')
hold on
semilogy(EbN0_RCU.',rho_RCU.','-')
semilogy(EbN0_NA.',rho_NA.','-.')
hold off
xlabel('SNR $E_b/N_0$')
ylabel('spectral efficiency $\rho$ [bit/s/Hz]')
grid


%% PRINT RESULT
% print -dpdf fig10a.pdf 
