%% 1) BUILD THE CHANNEL

% the channel
N = 128; % FFT size
T = 1e-9; % channel use period [s]
B = 1/(N*T);  % bandwidth [Hz] of each OFDM channel
gch(1) = 1; % channel samples
gch(10) = 1*exp(1i*2*pi*rand);
gch(17) = 0.7*exp(1i*2*pi*rand);
gch(24) = 0.3*exp(1i*2*pi*rand);
gch(35) = 0.1*exp(1i*2*pi*rand);
gch(70) = 0.01*exp(1i*2*pi*rand);
gch(N) = 0; % end channel
gch = gch*10^(-26.8/20); % 26.5dB attenuation
Gch = abs(fftshift(fft(gch))); % in frequency
f = (-N/2:N/2-1)*B/1e6; % [MHz]
sGch = sort(Gch,2,'descend');
sf = (0:N-1)*B/1e6; % [MHz]
% the power
P = 0.5; % [W]
% the noise
No = 2e-12; % noise level
sig2 = (No*B)./sGch.^2; % [W]

% plot channel
figure(1)
set(0,'defaulttextinterpreter','latex')
subplot(2,2,1)
plot(f, 20*log10(Gch))
axis([min(f) max(f) ylim])
xlabel('f [MHz]')
ylabel('$|G_{Ch}(f)|$ [dB]')
grid
subplot(2,2,2)
plot(sf, 20*log10(sGch))
axis([min(sf) max(sf) ylim])
xlabel('f [MHz]')
ylabel('reordered coeff. [dB]')
grid


%% 2) EVALUATE THE UPPER BOUND(S) ON RATE
disp('UPPER BOUND ON RATE')

Pe = 1e-5; % reference error probability
n = 2*N*[1,2,5,10,20,50,100,200,500,1000,1e4,1e5]; % packet lengths
clear C R_na R_ppv

% capacity
C = waterfilling_NA(inf,N,sig2,Pe,P);
% normal approximation
tic
for k = 1:length(n)
    R_na(k) = waterfilling_NA(n(k),N,sig2,Pe,P);
end
toc
% ppv meta-converse bound O(n^-2) approximation
tic
for k = 1:length(n)
    R_ppv(k) = waterfilling_PPV_On2(n(k),N,sig2,Pe,P);
end
toc

% show results
subplot(2,2,3)
semilogx(n/(2*N), 2*[ones(size(n))*C;R_ppv;R_na])
xlabel('\# of OFDM symbols ($n/K$)')
ylabel('upper bound on rate [bit/s/Hz]')
grid
legend('capacity','PPV','normal app.','Location','SouthEast')


%% 3) WATERFILLING EXAMPLE
disp('WATERFILLING EXAMPLE')

tic
[~, info_cap] = waterfilling_NA(inf,N,sig2,Pe,P);
toc
pow_cap = info_cap.pow;
tic
[~, info_ppv] = waterfilling_PPV_On2(n(1),N,sig2,Pe,P);
toc
pow_ppv = info_ppv.pow;
tic
[~, info_ppvb] = waterfilling_PPV_On3(n(1),N,sig2,Pe,P);
toc
pow_ppvb = info_ppvb.pow;

% show result
subplot(2,2,4)
const = 1e9/B; % [mW/MHz]
semilogy(sf, [sig2;sig2+pow_ppv;sig2+pow_cap;sig2+pow_ppvb]*const)
xlabel('f [MHz]')
ylabel('water filling level [mW/MHz]')
grid
legend({'$N_0/|G_{Ch}(f)|^2$','PPV $n=K=256\quad$','Capacity'},'Location','NorthWest','interpreter','latex')


%% PRINT RESULT
% print -dpdf fig2.pdf

